import os, sys

def is_running (pid, cmd = None):
	if cmd is None:
		cmd = os.path.split (sys.argv [0])[1]
		
	if os.name == "nt":
		import win32process, win32api, win32con, pywintypes
		HAS_WMI = True
		try: import wmi	
		except ImportError: HAS_WMI = False
		
		if pid not in win32process.EnumProcesses ():
			return False
	
		if HAS_WMI:
			cl = [p.CommandLine for p in wmi.WMI ().Win32_Process () if p.ProcessID == pid]
			if cl and cl [0].find (cmd) != -1:
				return True
			return False
												
		else:	
			try:
				handle = win32api.OpenProcess (win32con.PROCESS_QUERY_INFORMATION | win32con.PROCESS_VM_READ, 0, int (pid))
				exefilename = win32process.GetModuleFileNameEx (handle, 0)
				win32process.GetStartupInfo()
				if exefilename.lower ().find ("python.exe") != -1 or exefilename.lower ().find ("cmd.exe") != -1:
					return True
			except pywintypes.error: 
				# Windows service, Access is denied
				return False
	
	else:
		proc = "/proc/%s/cmdline" % pid
		if not os.path.isfile (proc):
			return False
		
		with open (proc) as f:
			exefilename = f.read ()		
		if exefilename.find (cmd) != -1:
			return True
		
	return False

if os.name == "nt":
	import win32pdh
	import win32process
	import win32event
	import pywintypes
					
	def timeout_execute (cmd, timeout = 0):
		if timeout == 0:
			timeout = win32event.INFINITE
			
		info  = win32process.CreateProcess(None, cmd, None, None, 0, 0, None, None, win32process.STARTUPINFO())
		subprocess = info [0]
		
		rc = win32event.WaitForSingleObject (subprocess, timeout)			
		
		if rc == win32event.WAIT_FAILED:	
			return -1
			
		if rc == win32event.WAIT_TIMEOUT:
			try:
				win32process.TerminateProcess (subprocess, 0)					
			except pywintypes.error:
				return -3
			return -2
		
		if rc == win32event.WAIT_OBJECT_0:
			return win32process.GetExitCodeProcess(subprocess)
	
	
	def get_child_pid (cpid):
		object = 'Process'
		items, instances = win32pdh.EnumObjectItems(None, None, object, 
		                                           win32pdh.PERF_DETAIL_WIZARD)
		instance_dict = {}
		for instance in instances:
			try:
				instance_dict[instance] = instance_dict[instance] + 1
			except KeyError:
				instance_dict[instance] = 0
		
		processinfos = []		
		for instance, max_instances in list(instance_dict.items()):
			for inum in range(max_instances+1):
				processinfo = []
				hq = win32pdh.OpenQuery()
				hcs = []
				for item in ['ID Process', 'Creating Process ID']:
					path = win32pdh.MakeCounterPath((None,object,instance,None,inum,item))
					hcs.append(win32pdh.AddCounter(hq,path))
				win32pdh.CollectQueryData(hq)
				processinfo.append (instance[:15].strip ())
				#print "%-15s\t" % (instance[:15]),
				for hc in hcs:
					type,val=win32pdh.GetFormattedCounterValue(hc,win32pdh.PDH_FMT_LONG)
					processinfo.append (int (val))
					#print "%5d" % (val),
					win32pdh.RemoveCounter(hc)
				#print
				win32pdh.CloseQuery(hq)
				processinfos.append (tuple (processinfo))
		
		def recusive (cpid):
			pids = []
			for name, pid, ppid in processinfos:
				if ppid == cpid:
					pids.append (pid)
			for pid in pids:
				pids += recusive (pid)		
			return pids	
		
		pids = recusive (cpid)	
		return pids

if __name__ == "__main__":
	print(get_child_pid (3172))
	




####
# Copyright 2000,2001 by Timothy O'Malley <timo@alum.mit.edu>
# 
#				All Rights Reserved
# 
# Permission to use, copy, modify, and distribute this software
# and its documentation for any purpose and without fee is hereby
# granted, provided that the above copyright notice appear in all
# copies and that both that copyright notice and this permission
# notice appear in supporting documentation, and that the name of
# Timothy O'Malley  not be used in advertising or publicity
# pertaining to distribution of the software without specific, written
# prior permission. 
# 
# Timothy O'Malley DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
# SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
# AND FITNESS, IN NO EVENT SHALL Timothy O'Malley BE LIABLE FOR
# ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
# WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
# PERFORMANCE OF THIS SOFTWARE. 
#
####

"""Timeout Socket

This module enables a timeout mechanism on all TCP connections.  It
does this by inserting a shim into the socket module.  After this module
has been imported, all socket creation goes through this shim.  As a
result, every TCP connection will support a timeout.

The beauty of this method is that it immediately and transparently
enables the entire python library to support timeouts on TCP sockets.
As an example, if you wanted to SMTP connections to have a 20 second
timeout:

	import timeoutsocket
	import smtplib
	timeoutsocket.setDefaultSocketTimeout(20)


The timeout applies to the socket functions that normally block on
execution:  read, write, connect, and accept.  If any of these 
operations exceeds the specified timeout, the exception Timeout
will be raised.

The default timeout value is set to None.  As a result, importing
this module does not change the default behavior of a socket.  The
timeout mechanism only activates when the timeout has been set to
a numeric value.  (This behavior mimics the behavior of the
select.select() function.)

This module implements two classes: TimeoutSocket and TimeoutFile.

The TimeoutSocket class defines a socket-like object that attempts to
avoid the condition where a socket may block indefinitely.  The
TimeoutSocket class raises a Timeout exception whenever the
current operation delays too long. 

The TimeoutFile class defines a file-like object that uses the TimeoutSocket
class.  When the makefile() method of TimeoutSocket is called, it returns
an instance of a TimeoutFile.

Each of these objects adds two methods to manage the timeout value:

	get_timeout()   -->  returns the timeout of the socket or file
	set_timeout()   -->  sets the timeout of the socket or file


As an example, one might use the timeout feature to create httplib
connections that will timeout after 30 seconds:

	import timeoutsocket
	import httplib
	H = httplib.HTTP("www.python.org")
	H.sock.set_timeout(30)

Note:  When used in this manner, the connect() routine may still
block because it happens before the timeout is set.  To avoid
this, use the 'timeoutsocket.setDefaultSocketTimeout()' function.

Good Luck!

"""

__version__ = "$Revision: 1.23 $"
__author__  = "Timothy O'Malley <timo@alum.mit.edu>"

#
# Imports
#
import ssl # fetched by Hans
import select, string
import socket
if not hasattr(socket, "_no_timeoutsocket"):
	_socket = socket.socket
else:
	_socket = socket._no_timeoutsocket


#
# Set up constants to test for Connected and Blocking operations.
# We delete 'os' and 'errno' to keep our namespace clean(er).
# Thanks to Alex Martelli and G. Li for the Windows error codes.
#
import os
if os.name == "nt":
	_IsConnected = ( 10022, 10056 )
	_ConnectBusy = ( 10035, )
	_AcceptBusy  = ( 10035, )
else:
	import errno
	_IsConnected = ( errno.EISCONN, )
	_ConnectBusy = ( errno.EINPROGRESS, errno.EALREADY, errno.EWOULDBLOCK )
	_AcceptBusy  = ( errno.EAGAIN, errno.EWOULDBLOCK )
	del errno
del os


#
# Default timeout value for ALL TimeoutSockets
#
_DefaultTimeout = None
def setDefaultSocketTimeout(timeout):
	global _DefaultTimeout
	_DefaultTimeout = timeout
def getDefaultSocketTimeout():
	return _DefaultTimeout

#
# Exceptions for socket errors and timeouts
#
Error = socket.error
class Timeout(Exception):
	pass


#
# Factory function
#
from socket import AF_INET, SOCK_STREAM
def timeoutsocket(family=AF_INET, type=SOCK_STREAM, proto=None):
	if family != AF_INET or type != SOCK_STREAM:
		if proto:
			return _socket(family, type, proto)
		else:
			return _socket(family, type)
	return TimeoutSocket( _socket(family, type), _DefaultTimeout )
# end timeoutsocket

#
# The TimeoutSocket class definition
#
class TimeoutSocket:
	"""TimeoutSocket object
	Implements a socket-like object that raises Timeout whenever
	an operation takes too long.
	The definition of 'too long' can be changed using the
	set_timeout() method.
	"""

	_copies = 0
	_blocking = 1
	
	def __init__(self, sock, timeout):
		self._sock	 = sock
		self._timeout  = timeout
	# end __init__

	def __getattr__(self, key):
		return getattr(self._sock, key)
	# end __getattr__

	def get_timeout(self):
		return self._timeout
	# end set_timeout

	def set_timeout(self, timeout=None):
		self._timeout = timeout
	# end set_timeout

	def setblocking(self, blocking):
		self._blocking = blocking
		return self._sock.setblocking(blocking)
	# end set_timeout

	def connect_ex(self, addr):
		errcode = 0
		try:
			self.connect(addr)
		except Error as why:
			errcode = why.errno
		return errcode
	# end connect_ex
		
	def connect(self, addr, port=None, dumbhack=None):
		# In case we were called as connect(host, port)
		if port != None:  addr = (addr, port)

		# Shortcuts
		sock	= self._sock
		timeout = self._timeout
		blocking = self._blocking
		# First, make a non-blocking call to connect
		try:
			sock.setblocking(0)
			sock.connect(addr)
			sock.setblocking(blocking)
			return
		except Error as why:
			# Set the socket's blocking mode back
			sock.setblocking(blocking)
			
			# If we are not blocking, re-raise
			if not blocking:
				raise
			
			# If we are already connected, then return success.
			# If we got a genuine error, re-raise it.
			errcode = why.errno
			if dumbhack and errcode in _IsConnected:
				return
			elif errcode not in _ConnectBusy:
				raise
			
		# Now, wait for the connect to happen
		# ONLY if dumbhack indicates this is pass number one.
		#   If select raises an error, we pass it on.
		#   Is this the right behavior?
		if not dumbhack:			
			r,w,e = select.select([], [sock], [], timeout)
			if w:
				return self.connect(addr, dumbhack=1)

		# If we get here, then we should raise Timeout
		raise Timeout("Attempted connect to %s timed out." % str(addr) )
	# end connect

	def accept(self, dumbhack=None):
		# Shortcuts
		sock	 = self._sock
		timeout  = self._timeout
		blocking = self._blocking

		# First, make a non-blocking call to accept
		#  If we get a valid result, then convert the
		#  accept'ed socket into a TimeoutSocket.
		# Be carefult about the blocking mode of ourselves.
		try:
			sock.setblocking(0)
			newsock, addr = sock.accept()
			sock.setblocking(blocking)
			timeoutnewsock = self.__class__(newsock, timeout)
			timeoutnewsock.setblocking(blocking)
			return (timeoutnewsock, addr)
		except Error as why:
			# Set the socket's blocking mode back
			sock.setblocking(blocking)

			# If we are not supposed to block, then re-raise
			if not blocking:
				raise
			
			# If we got a genuine error, re-raise it.
			errcode = why.errno
			if errcode not in _AcceptBusy:
				raise
			
		# Now, wait for the accept to happen
		# ONLY if dumbhack indicates this is pass number one.
		#   If select raises an error, we pass it on.
		#   Is this the right behavior?
		if not dumbhack:
			r,w,e = select.select([sock], [], [], timeout)
			if r:
				return self.accept(dumbhack=1)

		# If we get here, then we should raise Timeout
		raise Timeout("Attempted accept timed out.")
	# end accept

	def send(self, data, flags=0):
		sock = self._sock
		if self._blocking:
			r,w,e = select.select([],[sock],[], self._timeout)
			if not w:
				raise Timeout("Send timed out")
		return sock.send(data, flags)
	# end send

	def recv(self, bufsize, flags=0):
		sock = self._sock
		if self._blocking:
			r,w,e = select.select([sock], [], [], self._timeout)
			if not r:
				raise Timeout("Recv timed out")
		return sock.recv(bufsize, flags)
	# end recv

	def makefile(self, flags="r", bufsize=-1):
		self._copies = self._copies +1
		return TimeoutFile(self, flags, bufsize)
	# end makefile

	def close(self):
		if self._copies <= 0:
			self._sock.close()
		else:
			self._copies = self._copies -1
	# end close

# end TimeoutSocket


class TimeoutFile:
	"""TimeoutFile object
	Implements a file-like object on top of TimeoutSocket.
	"""
	
	def __init__(self, sock, mode="r", bufsize=4096):
		self._sock		  = sock
		self._bufsize	   = 4096
		if bufsize > 0: self._bufsize = bufsize
		if not hasattr(sock, "_inqueue"): self._sock._inqueue = ""

	# end __init__

	def __getattr__(self, key):
		return getattr(self._sock, key)
	# end __getattr__

	def close(self):
		self._sock.close()
		self._sock = None
	# end close
	
	def write(self, data):
		self.send(data)
	# end write

	def read(self, size=-1):
		_sock = self._sock
		_bufsize = self._bufsize
		while 1:
			datalen = len(_sock._inqueue)
			if datalen >= size >= 0:
				break
			bufsize = _bufsize
			if size > 0:
				bufsize = min(bufsize, size - datalen )
			buf = self.recv(bufsize)
			if not buf:
				break
			_sock._inqueue = _sock._inqueue + buf
		data = _sock._inqueue
		_sock._inqueue = ""
		if size > 0 and datalen > size:
			_sock._inqueue = data[size:]
			data = data[:size]
		return data
	# end read

	def readline(self, size=-1):
		_sock = self._sock
		_bufsize = self._bufsize
		while 1:
			idx = string.find(_sock._inqueue, "\n")
			if idx >= 0:
				break
			datalen = len(_sock._inqueue)
			if datalen >= size >= 0:
				break
			bufsize = _bufsize
			if size > 0:
				bufsize = min(bufsize, size - datalen )
			buf = self.recv(bufsize)
			if not buf:
				break
			_sock._inqueue = _sock._inqueue + buf

		data = _sock._inqueue
		_sock._inqueue = ""
		if idx >= 0:
			idx = idx + 1
			_sock._inqueue = data[idx:]
			data = data[:idx]
		elif size > 0 and datalen > size:
			_sock._inqueue = data[size:]
			data = data[:size]
		return data
	# end readline

	def readlines(self, sizehint=-1):
		result = []
		data = self.read()
		while data:
			idx = string.find(data, "\n")
			if idx >= 0:
				idx = idx + 1
				result.append( data[:idx] )
				data = data[idx:]
			else:
				result.append( data )
				data = ""
		return result
	# end readlines

	def flush(self):  pass

# end TimeoutFile


#
# Silently replace the socket() builtin function with
# our timeoutsocket() definition.
#
if not hasattr(socket, "_no_timeoutsocket"):
	socket._no_timeoutsocket = socket.socket
	socket.socket = timeoutsocket
del socket
socket = timeoutsocket
# Finis

def use_tsocket ():
	import socket
	if hasattr(socket, "_timeoutsocket"):
		socket._no_timeoutsocket = socket.socket
		socket.socket = socket._timeoutsocket
		del socket._timeoutsocket
	
def use_socket ():	
	import socket
	if hasattr(socket, "_no_timeoutsocket"):
		socket._timeoutsocket = socket.socket
		socket.socket = socket._no_timeoutsocket	
	


import os, sys
from . import pathtool, processutil

isCurrentProcess = processutil.is_running	
	
class PidFile:
	def __init__ (self, path):
		self.path = path
	
	def make (self):
		import os, sys		
		if self.isalive ():
			raise AssertionError("process is runnig. terminated.")
					
		pathtool.mkdir (self.path)
		pidfile = os.path.join (self.path, "pid")			
		f = open (pidfile, "w")
		f.write ("%s" % os.getpid ())
		f.close ()
	
	def remove (self):		
		pidfile = os.path.join (self.path, "pid")
		if os.path.isfile (pidfile):
			os.remove (pidfile)
		
	def getpid (self, match = None):
		pidfile = os.path.join (self.path, "pid")
		if os.path.isfile (pidfile):
			f = open (pidfile)
			pid = f.read ()
			f.close ()
			pid = int (pid)
			if isCurrentProcess (pid, match):
				return pid
		return None
			
	def isalive (self):
		return self.getpid () is not None and True or False
				

class Lock:
	def __init__ (self, home):
		self.home = home
		self.pidlock = None
		self.create_directory ()
	
	def create_directory (self):
		pathtool.mkdir (self.home)
	
	def get_pidlock (self):
		if self.pidlock is None:
			self.pidlock = PidFile (self.home)
		return self.pidlock
		
	def lock (self, name, msg = "do not delete"):
		if not self.islocked (name):
			f = open (os.path.join (self.home, "lock." + name), "w")
			f.write ("%s\n" % msg)
			f.close ()
		return 1

	def unlock (self, name):
		lock = os.path.join (self.home, "lock."  + name)
		if os.path.isfile (lock):
			os.remove (lock)
		return 1

	def unlockall (self):
		for file in os.listdir (self.home):
			s = file.find ("lock.")
			if  s != 0: continue
			os.remove (os.path.join (self.home, file))
		return 1

	def islocked (self, name):	
		try:
			return "lock."  + name in os.listdir (self.home)
		except (IOError, OSError):
			return False
	
	def isplocked (self, name):
		try:
			return [x for x in [lock.find ("lock." + name) for lock in os.listdir (self.home)] if x > -1]
		except (WindowsError, IOError, OSError):
			return False

	def locktime (self, name):
		lock = os.path.join (self.home, "lock."  + name)
		if os.path.isfile (lock):
			k = os.stat (lock)
			return k.st_mtime
		return 0

	def lockread (self, name):
		lock = os.path.join (self.home, "lock."  + name)
		if os.path.isfile (lock):
			f = open (lock)
			data = f.read ()
			f.close ()
			return data.strip ()
		return ""

	def locks (self, flt = ""):
		locks, errmsg = [], ''
		for file in os.listdir (self.home):
			if not file.startswith ("lock.%s" % flt):
				continue
			locks.append ((file [5:], self.locktime (file [5:])))
			if file [5:] == "panic":
				f = open (os.path.join (self.home, file))
				errmsg = f.read ()
				f.close ()
		return locks, errmsg


if __name__ == "__main__":
	f = Lock ("d:/")
	print(f.isplocked ("dup"))
	
	
	


"""
2008 added for asynchronous DNS query by Hans Roh
"""

import asyncore
import socket
import time
import types
import random
import types
from .pydns import Base, Type, Class, Lib, Opcode
import random

defaults = Base.defaults
Base.DiscoverNameServers ()

class async_dns (asyncore.dispatcher_with_send):
	zombie_timeout = 10
	
	def __init__ (self, servers, request, args, callback, logger, debug_level):
		self.servers = servers
		if not self.servers:
			raise OSError ("DNS Server Required")
		random.shuffle (self.servers)
		#self.servers = [("156.154.71.3", 513), ("156.154.71.9", 543)]
		self.addr = self.servers.pop (0)
		self.request = request
		self.callback = callback		
		self.args = args		
		self.logger = logger
		self.debug_level = debug_level
		self.qname = self.args ["name"].decode ("utf8")
		
		self.creation_time = time.time ()
		self.event_time = time.time ()
		self.ac_in_buffer = b""
		self.closed = False
		
		asyncore.dispatcher_with_send.__init__ (self)		
		self.create_socket (socket.AF_INET, socket.SOCK_STREAM)				
				
		try:
			self.connect (self.addr)
		except:
			self.handle_error ()
	
	def __repr__ (self):
		return "<async_dns: %s>" % self.qname
				
	def trace (self):
		self.logger.trace (self.qname)
	
	def log_info (self, line, level = 'info'):
		self.log ("[%s:%s] %s" % (level, self.qname, line))
	
	def log (self, line):
		self.logger (line)
	
	def create_socket (self, family, type):
		sock_class = socket.socket			
		self.family_and_type = family, type
		self.socket = sock_class (family, type)
		self.socket.setblocking (0)
		self._fileno = self.socket.fileno ()
		self._timeouted = 0
		self.add_channel()
		
	def handle_error (self):
		if self.debug_level: 
			self.trace ()
		self.close ()
	
	def handle_timeout (self):
		if self.debug_level: 
			self.log_info ('DNS query timeout %s' % self.callback, 'error')
			self._timeouted = 1
		self.handle_close ()
					
	def handle_connect (self):	
		self.send (self.request)
		
	def handle_write (self):
		if not self.closed:
			self.socket.shutdown (1)
		
	def handle_read (self):
		try:
			data = self.recv (4096)
		except BlockingIOError:
			return
		except OSError as why:
			self.handle_close()
			return
		self.ac_in_buffer += data		
		
	def handle_expt (self):
		self.handle_close ()
	
	def close (self):
		if self.closed:
			return
		self.closed = True	
		asyncore.dispatcher_with_send.close (self)		
		self.callback (self.servers, self.request, self.args, self.ac_in_buffer, self._timeouted)
			
	def handle_close (self):
		self.close ()
	
	
class Request:
	def __init__(self, logger, server = [], debug_level = 1):
		if type (server) is bytes:
			server = [server]
		
		defaults ["server"] += server		
		self.logger = logger
		self.debug_level = debug_level		
		
	def argparse (self, name, args):
		args['name']=name
		for i in list(defaults.keys()):
			if i not in args:
				if i == "server": 
					args[i]=defaults[i][:]
				else:	
					args[i]=defaults[i]
				
		if type (args['server']) == bytes:
			args ['server'] = [args['server']]
			
		return args
		
	def req (self, name, **args):
		name = name.encode ("utf8")
		args = self.argparse (name, args)
		
		protocol = args ['protocol']
		port = args ['port']
		opcode = args ['opcode']
		rd = args ['rd']
		server = args ['server'][:]
		#server = ['127.0.0.1:6000']
		
		if type(args['qtype']) in (bytes, str):
			try:
				qtype = getattr (Type, args ['qtype'].upper ())
			except AttributeError:
				raise Base.DNSError('%s unknown query type' % name)
				
		else:
			qtype = args ['qtype']
			
		qname = args ['name']
		
		#print 'QTYPE %d(%s)' % (qtype, Type.typestr(qtype))
		m = Lib.Mpacker()
		# jesus. keywords and default args would be good. TODO.
		m.addHeader(0,
			  0, opcode, 0, 0, rd, 0, 0, 0,
			  1, 0, 0, 0)
		
		m.addQuestion (qname, qtype, Class.IN)
		request = m.getbuf ()
		request = Lib.pack16bit (len(request)) + request
		
		server = [(x, args ["port"]) for x in server]
		async_dns (server, request, args, self.processReply, self.logger, self.debug_level)
			
	def processReply (self, server, request, args, data, timeouted):
		if timeouted:
			# for retrying
			answers = []
			
		else:	
			# do not query for 1 minutes
			not_found = [{'err': True, "name": args ['name'].decode ('utf8'), "data": None, "typename": args ["qtype"], 'ttl': 60}]
				
			try:
				if not data:
					raise Base.DNSError('%s, no working nameservers found' % args ['name'])
			
				if args ["protocol"] == "tcp":
					header = data [:2]
					if len (header) < 2:
						raise Base.DNSError('%s, EOF' % args ['name'])
					count = Lib.unpack16bit(header)		
					reply = data [2: 2 + count]
					if len (reply) != count:
						raise Base.DNSError('%s, incomplete reply' % args ['name'])
				
				else:
					reply = data
				
			except:					
				if server:
					async_dns (server, request, args, self.processReply, self.logger, self.debug_level)
					return
					
				else:
					reply = None
			
			if reply is None:
				answers = not_found
				
			else:
				try:	
					u = Lib.Munpacker(reply)
					r = Lib.DnsResult(u, args)
					r.args = args
					answers = r.answers
				except:
					self.logger.trace ()
					answers = not_found
					
			if not answers:
				answers = not_found
		
		callback = args.get ("callback", None)
		#self.logger ('DNS callback %s' % callback)
		if callback:
			if type (callback) != type ([]):
				callback = [callback]
			for cb in callback:
				cb (answers)
			

if __name__	== "__main__":
	from aquests.lib import logger
	import pprint
	f = Request (logger.screen_logger ())
	f.req ("www.yahoo.com", protocol = "tcp", callback = pprint.pprint, qtype="a")
	f.req ("www.hungryboarder.com", protocol = "tcp", callback = pprint.pprint, qtype="a")
	f.req ("www.alexa.com", protocol = "tcp", callback = pprint.pprint, qtype="a")
	f.req ("www.google.com", protocol = "tcp", callback = pprint.pprint, qtype="mx")
	asyncore.loop (timeout = 1)
	
	



from ..http import response

class Response (response.Response):
	def __init__ (self, request, code, msg, opcode = None, data = None):
		self.request = request
		self.code = code
		self.msg = msg
		self.__data = []
		if opcode:
			self.__data.append ((opcode, data))		
		self.version = "1.1"		
		
	def add_message (self, opcode, data = None):
		self.__data.append ((opcode, data))
		
	@property
	def content (self):
		if not self.__data:
			return None
		return self.__data [0]
		
	@property
	def data (self):
		if not self.__data:
			return None
		return self.content [0][1]
	
	@property
	def opcode (self):
		if not self.__data:
			return None
		return self.content [0][0]
	
	
	@property
	def headers (self):
		return {}
		
	def done (self):
		pass
	


from ...protocols.http import base_request_handler
from h2.connection import H2Connection, GoAwayFrame
from h2.exceptions import ProtocolError, NoSuchStreamError, StreamClosedError, FlowControlError
from h2.events import DataReceived, ResponseReceived, StreamEnded, ConnectionTerminated, StreamReset, WindowUpdated, RemoteSettingsChanged
from h2.errors import ErrorCodes
from h2.settings import SettingCodes
from ...lib import producers
from .producers import h2frame_producer, h2header_producer
from ..http import respcodes
from ..http import response as http_response

from ...client import asynconnect
import threading
try:
	from cStringIO import StringIO as BytesIO
except ImportError:
	from io import BytesIO

class FakeAsynConnect:
	collector = None
	def __init__ (self, asyncon, logger):
		self.asyncon = asyncon
		self.logger = logger
	
	def get_proto (self):
		return "h2"
		
	def is_proxy (self):
		return False
		
	def handle_close (self, *args, **karg):
		pass
		
	def handle_error (self):
		self.logger.trace ()
	
	def isconnected (self):
		return True
		
	def push (self, *args, **karg):	
		pass
	
	def set_terminator (self, *args, **karg):
		pass		
			
	def get_terminator (self):
		pass
	
	def begin_tran (self, *args, **karg):	
		pass
	
	def end_tran (self, *args, **karg):	
		pass	
	
	def collect_incoming_data (self):
		if self.collector:
			self.collector.collect_incoming_data ()
	
	def disconnect (self):
		pass
			
	
class RequestHandler (base_request_handler.RequestHandler):
	# HTTP2 Adaptor Class	
	# this class rely: asyncon - this class - http_reqyest_handler
	def __init__ (self, handler):
		self.asyncon = handler.asyncon
		self.request = handler.request
		base_request_handler.RequestHandler.__init__ (self, handler.request.logger)
		#self.asyncon.set_timeout (60, 60)
		self.lock = handler.asyncon.lock # pool lock
		self._ssl = handler._ssl
		
		self._clock = threading.RLock () # conn lock
		self._llock = threading.RLock () # local lock
		self.fakecon = FakeAsynConnect (self.asyncon, self.logger)		
		
		self.initiate ()		
		is_upgrade = not (self._ssl or self.request.initial_http_version == "2.0")			
		if is_upgrade:
			self.conn.initiate_upgrade_connection()
			self.conn.update_settings({SettingCodes.ENABLE_PUSH: 0})
			# assume 1st request's stream_id = 1
			self.add_request (1, handler)
			self._send_stream_id = 1 # nexet request stream_id will be 3
		else:		    
			self.conn.initiate_connection()
			
		self.send_data ()
		self.asyncon.set_terminator (9)
		
		if not is_upgrade:			
			self.handle_request (handler)
		else:
			self.asyncon.set_active (False)
	
	def initiate (self):	
		self.asyncon.handler = self
		self.asyncon.use_sendlock ()
		self._send_stream_id = -1
		self.requests = {}
		self.conn = H2Connection ()
		self.buf = b""
		self.rfile = BytesIO ()
		self.frame_buf = self.conn.incoming_buffer
		self.frame_buf.max_frame_size = self.conn.max_inbound_frame_size		
		self.data_length = 0
		self.current_frame = None
	
	def working (self):
		return self.jobs () and True or False
	
	def jobs (self):
		with self._llock:
			return len (self.requests)
		
	def go_away (self, errcode = 0, msg = None):
		with self._plock:
			self.conn.close_connection (errcode, msg)
		self.send_data ()
		self.channel.close_when_done ()	
		
	def enter_shutdown_process (self, err = 0):
		self.go_away (ErrorCodes.NO_ERROR)
		self.asyncon.handler = None # unlink
				
	def add_request (self, stream_id, handler):		
		handler.asyncon = self.fakecon
		with self._llock:
			self.requests [stream_id] = handler
				
	def get_new_stream_id (self):
		with self._llock:
			self._send_stream_id += 2
			stream_id = self._send_stream_id
		return stream_id
	
	def send_data (self):
		with self._clock:
			data_to_send = self.conn.data_to_send ()		
		if data_to_send:
			self.asyncon.push (data_to_send)
							
	def handle_request (self, handler):
		self.request = handler.request
		stream_id = self.get_new_stream_id ()		
		self.add_request (stream_id, handler)		
		self.asyncon.set_active (False)		
		
		headers, content_encoded = handler.get_request_header ("2.0", False)
		payload = handler.get_request_payload ()
		producer = None
		if payload:
			if type (payload) is bytes:
				producer = producers.globbing_producer (
					producers.simple_producer (payload)
				)
			else:
				# multipart, grpc_producer 
				producer = producers.globbing_producer (payload)
		
		header = h2header_producer (stream_id, headers, producer, self.conn, self._clock)		
		self.asyncon.push (header)				
		if producer:
			payload = h2frame_producer (stream_id, 0, 1, producer, self.conn, self._clock)
			self.asyncon.push (payload)
	
	def increment_flow_control_window (self, cl, stream_id = 0):
		if not stream_id:
			with self._clock:
				self.conn.increment_flow_control_window (cl)
				self.send_data ()				
		else:	
			with self._clock:
				try: self.conn.increment_flow_control_window (cl, stream_id)
				except StreamClosedError: pass
				else: self.send_data ()	
					
	def collect_incoming_data (self, data):
		if not data:  return
		if self.data_length:
			self.rfile.write (data)
		else:
			self.buf += data
	
	def connection_closed (self, why, msg):
		with self._llock:
			requests = list (self.requests.items ())
		for stream_id, h in requests:
			h.response = http_response.FailedResponse (720, respcodes.get (720), h.request)			
			h.handle_callback ()
		with self._llock:
			self.requests = {}

	def found_terminator (self):
		buf, self.buf = self.buf, b""		
		events = None
		if self.data_length:			
			events = self.set_frame_data (self.rfile.getvalue ())				
			self.data_length = 0
			self.current_frame = None
			self.rfile.seek (0)
			self.rfile.truncate ()
			self.asyncon.set_terminator (9) # for frame header
						
		elif buf:
			self.current_frame, self.data_length = self.frame_buf._parse_frame_header (buf)
			self.frame_buf.max_frame_size = self.data_length
			if self.data_length == 0:
				events = self.set_frame_data (b'')
			self.asyncon.set_terminator (self.data_length == 0 and 9 or self.data_length)	# next frame header
			
		else:
			raise ProtocolError ("Frame decode error")
		
		if events:
			self.handle_events (events)	
	
	def set_frame_data (self, data):
		if not self.current_frame:
			return []
		self.current_frame.parse_body (memoryview (data))				
		self.current_frame = self.frame_buf._update_header_buffer (self.current_frame)
		with self._clock:
			events = self.conn._receive_frame (self.current_frame)
		return events
	
	def get_handler (self, stream_id):
		h = None
		with self._llock:
			try: h =	self.requests [stream_id]
			except KeyError: pass
		return h		
	
	def remove_handler (self, stream_id):
		with self._llock:
			try: del	self.requests [stream_id]
			except KeyError: pass
		
	def handle_response (self, stream_id, headers):
		# to HTTP/1.1 header
		jheaders = []
		for k, v in headers:
			k = k.decode ("utf8")
			v = v.decode ("utf8")
			if k == ":status":
				jheaders.insert (0, "HTTP/2.0 " + v + " " + respcodes.get (int (v), "Undefined Error"))
			else:
				jheaders.append (k + ": " + v)
		
		h = self.get_handler (stream_id)
		if h:
			h.collect_incoming_data ("\r\n".join (jheaders).encode ("utf8"))
			h.found_terminator ()
		
		# finally, make inactive on post, put request 
		self.asyncon.set_active (False)
		
	def handle_events (self, events):
		for event in events:
			#print (event)
			if isinstance(event, ResponseReceived):
				self.handle_response (event.stream_id, event.headers)
					
			elif isinstance(event, StreamReset):
				if event.remote_reset:				
					h = self.get_handler (event.stream_id)
					if h:
						h.response = http_response.FailedResponse (721, respcodes.get (721), h.request)
						h.handle_callback ()
						self.remove_handler (event.stream_id)
					
			elif isinstance(event, ConnectionTerminated):
				self.asyncon.handle_close (721, "HTTP2 Connection Terminated")
				
			elif isinstance (event, DataReceived):
				h = self.get_handler (event.stream_id)
				if h:
					h.collect_incoming_data (event.data)
					rfcw = self.conn.remote_flow_control_window (event.stream_id)
					if rfcw <= 16384:
						self.increment_flow_control_window (1048576, event.stream_id)
			
			elif isinstance (event, RemoteSettingsChanged):
				try:
					iws = event.changed_settings [SettingCodes.INITIAL_WINDOW_SIZE].new_value
				except KeyError:
					pass
				else:		
					self.increment_flow_control_window ((2 ** 31 - 1) - iws)
				
			elif isinstance(event, StreamEnded):
				h = self.get_handler (event.stream_id)
				if h:
					self.remove_handler (event.stream_id)
					h.found_terminator ()					
		self.send_data ()
		


import asyncore
import multiprocessing
import time
import sys
import os
from . import pathtool
import types
import codecs

PY_MAJOR_VERSION = sys.version_info.major

def trace ():
	(file,fun,line), t, v, tbinfo = asyncore.compact_traceback()
	try:
		v = str (v)
	except:	
		v = repr (v)
		
	return "%s %s Traceback: %s" % (t, v, tbinfo)
		
def now (detail = 1):
	if detail: return time.strftime("%Y.%m.%d %H:%M:%S", time.localtime(time.time()))
	else: return time.strftime("%Y%m%d", time.localtime(time.time()))

def ago (t):
	diff = time.time () - t
	if diff < 3600:
		return "%2.1f minutes ago" % (diff/60)
	elif diff < 3600	 * 24:
		return "%2.1f hours ago" % (diff/3600)
	elif diff < 3600	 * 72:
		return "%2.1f days ago" % (diff/(3600 * 24))	
	return time.strftime ("%Y-%m-%d", time.localtime (t))

	
class base_logger:
	def __init__(self, out, cacheline = 100, flushnow = 0):
		self.out = out
		self.cacheline = cacheline
		self.flushnow = flushnow
		self.lock = multiprocessing.Lock()
		self.filter = []
		self.__cache = []		
	
	def set_filter (self, *types):	
		self.filter = types
		
	def __call__(self, line, type="info", name=""):
		return self.log (line, type, name)
	
	def write (self, line, type="", name=""):
		return self.log (line, type, name)
	
	def cleanup (self):			
		self.close ()
		
	def flush (self): 
		try:
			self.out.flush ()
		except:
			pass	
	
	def cache (self, line):
		if self.cacheline:
			self.__cache.insert (0, line)
			self.__cache = self.__cache [:self.cacheline]
	
	def tag (self, type, name):
		try:
			name = str (name)
		except UnicodeEncodeError:
			pass
			
		tag = ""
		if type: 
			tag = "[%s%s] " % (type, name and ":" + name or "")
		return tag
		
	def log (self, line, type = "info", name = ""):
		line = str (line).strip ()
		line = "%s %s%s\n" % (now(), self.tag (type, name), line)
		
		if self.filter and type not in self.filter:
			return line
		
		self.lock.acquire ()	
		try:
			self.out.write (line)
		except UnicodeEncodeError:			
			self.out.write (repr (line.encode ("utf8")))
		if self.flushnow: self.flush ()
		self.cache (line)
		self.lock.release ()
		return line
		
	def close (self): 
		self.out.close ()
	
	def trace (self, name = ''):
		return self.log (trace (), "expt", name)	
	traceback = trace	
	
	def read (self):
		return self.__cache
		
	
class screen_logger (base_logger):
	def __init__ (self, cacheline = 200, flushnow = 1):
		base_logger.__init__(self, sys.stdout, cacheline, flushnow)
	
	def log (self, line, type = "info", name = ""):
		if type.startswith ("expt"):
			line = trace ().replace ("] [", "\n  - ")
			line = line.replace ("Traceback: [", "\n  -----------\n  + Traceback\n  ===========\n  - ")
			line = line [:-1] + "\n  -----------"
		base_logger.log (self, line, type, name)			
	
	def close (self): 
		pass


class null_logger (screen_logger):
	def log (self, line, type = "info", name = ""): 
		pass
		

class pipe_logger (screen_logger): 
	def __init__ (self):
		screen_logger.__init__(self, 0, 1)
	
	def log (self, line, type = "info", name = ""):		
		# do not write datetime
		if isinstance (line, str):
			line = line.encode ("ascii", "ignore")
			
		line = str (line).strip ()
		line = "%s%s\n" % (self.tag (type, name), line)
		
		if self.filter and type not in self.filter:
			return line
			
		self.lock.acquire ()
		self.out.write (line)
		if self.flushnow: self.flush ()
		self.cache (line)
		self.lock.release ()
		return line


class rotate_logger (base_logger):
	PRESERVE_MAX = 30
	def __init__(self, base, surfix = '', freq = "daily", cacheline = 200, flushnow = 0):
		self.base = base
		self.surfix = surfix
		self.freq = freq
		
		pathtool.mkdir (base)
		self.file = "%s/%s.log" % (self.base, self.surfix)
		
		base_logger.__init__ (self, codecs.open (self.file, "a", "utf8"), cacheline, flushnow)
				
		self.cv = multiprocessing.Condition (multiprocessing.RLock())
		self.using = 0
		self.numlog = 0
		self.maintern ()
		self.rotate_when = self.get_next_rotate (self.freq)
		
	def maybe_rotate (self):
		if self.freq and time.time() > self.rotate_when:
			self.rotate()
			self.rotate_when = self.get_next_rotate (self.freq)
		        
	def get_next_rotate (self, freq = "daily"):
		(yr, mo, day, hr, min, sec, wd, jday, dst) = time.localtime(time.time())
		if freq == 'daily':
			return time.mktime((yr,mo,day+1, 0,0,0, 0,0,-1))
			#return time.mktime((yr,mo,day, hr,min,sec+10, 0,0,-1))
		elif freq == 'weekly':
			return time.mktime((yr,mo,day-wd+7, 0,0,0, 0,0,-1))
		elif freq == 'monthly':
			return time.mktime((yr,mo+1,1, 0,0,0, 0,0,-1))
		else:
			return time.mktime((yr,mo,day+1, 0,0,0, 0,0,-1))
				
	def maintern (self):
		dlogs = []
		for file in os.listdir (self.base):
			if file.find (self.surfix + "-") != 0: continue 			
			dlogs.append (file) 
		
		if len (dlogs) <= self.PRESERVE_MAX: return
		dlogs.sort ()
		for file in dlogs [:-self.PRESERVE_MAX]:
			try: os.remove (os.path.join (self.base, file))
			except: pass
		
	def rotate (self):
		self.cv.acquire ()
		try:
			self.out.close ()		
			(yr, mo, day, hr, min, sec, wd, jday, dst) = time.localtime(time.time())
			newfile = "%s/%s-%04d%02d%02d.log" % (self.base, self.surfix, yr, mo, day)
			if os.path.isfile (newfile):
				 newfile = "%s/%s-%04d%02d%02d-%02d%02d%02d.log" % (self.base, self.surfix, yr, mo, day,hr, min, sec)		
			try:
				os.rename (self.file, newfile)
			except:
				self.out = codecs.open (self.file, "a", "utf8")
			else:	
				self.out = codecs.open (self.file, "w", "utf8")
			self.maintern ()
			
		finally:			
			self.cv.release ()		
	
	def close (self):
		self.cv.acquire ()
		while self.using:
			self.cv.wait ()			
		try:
			base_logger.close (self)	
		finally:	
			self.cv.release ()
			
	def log (self, line, type="info", name=""):
		try:
			line = str (line)
		except UnicodeEncodeError:
			pass
		
		lline = "%s %s%s\n" % (now(), self.tag (type, name), line)		
		if self.filter and type not in self.filter:
			return lline
		
		self.cv.acquire ()
		self.using = 1		
		try:
			self.out.write (lline)
			if self.flushnow: self.flush ()
			self.cache (line)
		except:
			self.out.write ("%s %%s\n", (now(), self.tag (type, name), repr (line)))
			
		self.using = 0
		self.numlog += 1
		numlog = self.numlog
		self.cv.notify_all ()
		self.cv.release ()
		
		if numlog % 1000 == 0:
			self.maybe_rotate ()
			
		return line		


class file_logger (rotate_logger):
	def __init__ (self, base, surfix = '', *arg, **karg):
		rotate_logger.__init__ (self, base, surfix)
		
	
class multi_logger (base_logger):
	def __init__(self, loggers = [], cacheline = 100, flushnow = 0):
		self.loggers = []
		for logger in loggers:
			self.loggers.append (logger)
			
		base_logger.__init__ (self, None, cacheline, flushnow)
	
	def rotate (self):
		for logger in self.loggers:
			hasattr (logger, 'rotate') and logger.rotate ()
			
	def log (self, line, type="info", name=""):
		if self.filter and type not in self.filter:
			return line
		
		for logger in self.loggers:
			_lline = logger.log (line, type, name)	
				
		self.cache (_lline)
		
	def add_logger (self, logger):
		self.loggers.append (logger)		
		
	def close (self):
		for logger in self.loggers:
			logger.close ()




if __name__ == "__main__":
	l = file_logger ("x:/test/", "test")			
	for i in range (300):
		l.write ("asdsadasda")
	l.close ()
	



from ...lib import producers, compressors
import struct
	
def get_messages (fp):
	decompressor = None	
	msgs = []
	byte = fp.read (1)
	while byte:
		iscompressed = struct.unpack ("!B", byte) [0]
		length = struct.unpack ("!I", fp.read (4)) [0]
		msg = fp.read (length)
		msg = decode_message (msg, iscompressed)
		byte = fp.read (1)
		msgs.append (msg)
	return msgs


def decode_message (msg, iscompressed):	
	if iscompressed:
		decompressor = compressors.GZipDecompressor ()
		msg = decompressor.deconpress (msg)	+ decompressor.flush ()
	return msg
	
